@@ -1,24 +1,22 @@
+#![no_std]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
+#![doc(
+    html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
+    html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
+    html_root_url = "https://docs.rs/pkcs5/0.5.0-pre"
+)]
+#![forbid(unsafe_code, clippy::unwrap_used)]
+#![warn(missing_docs, rust_2018_idioms, unused_qualifications)]
 
 //! # Usage
 //!
 //! The main API for this crate is the [`EncryptionScheme`] enum, which impls
-//! the [`Decodable`][`der::Decodable`] and [`Encodable`] traits from the
-//! [`der`] crate, and can be used for decoding/encoding PKCS#5
-//! [`AlgorithmIdentifier`] fields.
+//! the [`Decode`] and [`Encode`] traits from the [`der`] crate, and can be
+//! used for decoding/encoding PKCS#5 [`AlgorithmIdentifier`] fields.
 //!
 //! [RFC 8018]: https://tools.ietf.org/html/rfc8018
 
-#![no_std]
-#![cfg_attr(docsrs, feature(doc_cfg))]
-#![doc(
-    html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
-    html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
-    html_root_url = "https://docs.rs/pkcs5/0.4.0"
-)]
-#![forbid(unsafe_code, clippy::unwrap_used)]
-#![warn(missing_docs, rust_2018_idioms, unused_qualifications)]
-
 #[cfg(all(feature = "alloc", feature = "pbes2"))]
 extern crate alloc;
 
@@ -31,7 +29,7 @@ pub use crate::error::{Error, Result};
 pub use der::{self, asn1::ObjectIdentifier};
 pub use spki::AlgorithmIdentifier;
 
-use der::{Decodable, Decoder, Encodable, Encoder, Length, Tag};
+use der::{Decode, DecodeValue, Encode, Header, Reader, Sequence, Tag};
 
 #[cfg(all(feature = "alloc", feature = "pbes2"))]
 use alloc::vec::Vec;
@@ -44,7 +42,7 @@ pub enum EncryptionScheme<'a> {
     /// Password-Based Encryption Scheme 1 as defined in [RFC 8018 Section 6.1].
     ///
     /// [RFC 8018 Section 6.1]: https://tools.ietf.org/html/rfc8018#section-6.1
-    Pbes1(pbes1::Parameters),
+    Pbes1(pbes1::Algorithm),
 
     /// Password-Based Encryption Scheme 2 as defined in [RFC 8018 Section 6.2].
     ///
@@ -121,9 +119,9 @@ impl<'a> EncryptionScheme<'a> {
     }
 
     /// Get [`pbes1::Parameters`] if it is the selected algorithm.
-    pub fn pbes1(&self) -> Option<&pbes1::Parameters> {
+    pub fn pbes1(&self) -> Option<&pbes1::Algorithm> {
         match self {
-            Self::Pbes1(params) => Some(params),
+            Self::Pbes1(alg) => Some(alg),
             _ => None,
         }
     }
@@ -137,39 +135,27 @@ impl<'a> EncryptionScheme<'a> {
     }
 }
 
-impl<'a> Decodable<'a> for EncryptionScheme<'a> {
-    fn decode(decoder: &mut Decoder<'a>) -> der::Result<Self> {
-        AlgorithmIdentifier::decode(decoder)?.try_into()
+impl<'a> DecodeValue<'a> for EncryptionScheme<'a> {
+    fn decode_value<R: Reader<'a>>(decoder: &mut R, header: Header) -> der::Result<Self> {
+        AlgorithmIdentifier::decode_value(decoder, header)?.try_into()
     }
 }
 
-impl<'a> Encodable for EncryptionScheme<'a> {
-    fn encoded_len(&self) -> der::Result<Length> {
+impl<'a> Sequence<'a> for EncryptionScheme<'a> {
+    fn fields<F, T>(&self, f: F) -> der::Result<T>
+    where
+        F: FnOnce(&[&dyn Encode]) -> der::Result<T>,
+    {
         match self {
-            Self::Pbes1(pbes1) => pbes1.encoded_len(),
-            Self::Pbes2(pbes2) => {
-                (pbes2::PBES2_OID.encoded_len()? + pbes2.encoded_len()?)?.for_tlv()
-            }
-        }
-    }
-
-    fn encode(&self, encoder: &mut Encoder<'_>) -> der::Result<()> {
-        match self {
-            Self::Pbes1(pbes1) => pbes1.encode(encoder),
-            Self::Pbes2(pbes2) => {
-                let seq_len = (pbes2::PBES2_OID.encoded_len()? + pbes2.encoded_len()?)?;
-                encoder.sequence(seq_len, |seq| {
-                    seq.encode(&pbes2::PBES2_OID)?;
-                    seq.encode(pbes2)
-                })
-            }
+            Self::Pbes1(pbes1) => f(&[&pbes1.oid(), &pbes1.parameters]),
+            Self::Pbes2(pbes2) => f(&[&pbes2::PBES2_OID, pbes2]),
         }
     }
 }
 
-impl<'a> From<pbes1::Parameters> for EncryptionScheme<'a> {
-    fn from(params: pbes1::Parameters) -> EncryptionScheme<'a> {
-        Self::Pbes1(params)
+impl<'a> From<pbes1::Algorithm> for EncryptionScheme<'a> {
+    fn from(alg: pbes1::Algorithm) -> EncryptionScheme<'a> {
+        Self::Pbes1(alg)
     }
 }
 
@@ -184,13 +170,12 @@ impl<'a> TryFrom<AlgorithmIdentifier<'a>> for EncryptionScheme<'a> {
 
     fn try_from(alg: AlgorithmIdentifier<'a>) -> der::Result<EncryptionScheme<'_>> {
         if alg.oid == pbes2::PBES2_OID {
-            if let Some(params) = alg.parameters {
-                pbes2::Parameters::try_from(params).map(Into::into)
-            } else {
-                Err(Tag::OctetString.value_error())
+            match alg.parameters {
+                Some(params) => pbes2::Parameters::try_from(params).map(Into::into),
+                None => Err(Tag::OctetString.value_error()),
             }
         } else {
-            pbes1::Parameters::try_from(alg).map(Into::into)
+            pbes1::Algorithm::try_from(alg).map(Into::into)
         }
     }
 }
